function report_crm_task(reporting) {
  "use strict";

  var model = openerp;

  reporting.ReportCrmTaskWidget = reporting.Base.extend({
    template: 'ReportCrmTask',
    content: [],
    rowsData: [],
    modules: ['crm', 'eiru_crm'],
    events: {
      'click #toolbar > button': 'clickOnAction',
      'click #generate': 'fetchGenerate',
      'click-row.bs.table #table ': 'clickAnalysisDetail',
      'change #current-date': 'ShowDateRange',
      'change #current-store': 'updateUser',
    },

    init: function(parent) {
      this._super(parent);
    },

    start: function() {
      var table = this.$el.find('#table');
      table.bootstrapTable({
        data: self.rowsData
      });
      var date = new model.eiru_reports.ReportDatePickerWidget(self);
      date.fecthFecha();
      this.fetchInitial();
    },

    // Redirecionar
    renderReport: function() {
      var self = this;
      var container = this.$el.closest('.oe_form_sheet.oe_form_sheet_width');
      this.$el.closest('.report_view').remove();
      container.find('.report_view').show({
        effect: 'fade',
        duration: 200,
      });
    },
    // Verificar el modelo
    checkModel: function(model) {
      var self = this;
      return _.filter(self.modules, function(item) {
        return item.name === model
      });
    },
    // Lanzar el mensaje
    showMensaje: function(modelos) {
      var self = this;
      $("#dialog").dialog({
        autoOpen: true,
        resizable: false,
        modal: true,
        title: 'Atención',
        width: 500,
        open: function() {
          $(this).html('Reporte in-disponible, contacte con el administrador del sistema ref : ' + modelos);
        },
        show: {
          effect: "fade",
          duration: 200
        },
        hide: {
          effect: "fade",
          duration: 200
        },
        buttons: {
          Aceptar: function() {
            $(this).dialog('close');
            self.renderReport()
          }
        }
      });
      return
    },

    valorNull: function(dato) {
      var valor = "";
      if (dato) {
        valor = dato;
      }
      return valor;
    },

    ShowDateRange: function() {
      var self = this;
      var date = self.$el.find('#current-date').val();
      if (date == 'range') {
        self.$el.find('.datepicker').css('display', 'block');
      }
      if (date != 'range') {
        self.$el.find('.datepicker').css('display', 'none');
      }
    },

    clickAnalysisDetail: function(e, row, $element, field) {
      if (field == 'partner') {
        this.do_action({
          name: "Cliente",
          type: 'ir.actions.act_window',
          res_model: "res.partner",
          views: [
            [false, 'form']
          ],
          target: 'new',
          domain: [
            ['id', '=', row.partner_id]
          ],
          context: {},
          flags: {
            'form': {
              'action_buttons': false,
              'options': {
                'mode': 'view'
              }
            }
          },
          res_id: row.partner_id,
        });
      }
      if (field === 'activity') {
        this.do_action({
          name: "Registro de Tareas",
          type: 'ir.actions.act_window',
          res_model: "crm.task",
          views: [
            [false, 'form']
          ],
          target: 'new',
          domain: [
            ['id', '=', row.id]
          ],
          context: {},
          flags: {
            'form': {
              'action_buttons': false,
              'options': {
                'mode': 'view'
              }
            }
          },
          res_id: row.id,
        });
      }
      e.stopImmediatePropagation();
    },

    fetchInitial: function() {
      var self = this;
      self.fecthIrModuleModule().then(function(IrModuleModule) {
        return IrModuleModule;
      }).then(function(IrModuleModule) {
        self.IrModuleModule = IrModuleModule;
        return self.fetchResCompany();
      }).then(function(ResCompany) {
        self.ResCompany = ResCompany;
        if (ResCompany.length > 1) {
          self.$el.find('#current-company').append('<option value="9999999">Todas las empresas</option>');
          _.each(ResCompany, function(item) {
            self.$el.find('#current-company').append('<option value="' + item.id + '">' + item.name + '</option>');
          });
        } else {
          self.$el.find('.company').css('display', 'none');
        }
        return self.fetchResStore();
      }).then(function(ResStore) {
        self.ResStore = ResStore;
        if (ResStore.length > 1) {
          self.$el.find('#current-store').append('<option value="9999999">Todas las sucursales</option>');
          _.each(ResStore, function(item) {
            self.$el.find('#current-store').append('<option value="' + item.id + '">' + item.name + '</option>');
          });
        } else {
          self.$el.find('.store').css('display', 'none');
        }
        return self.fetchResUser();
      }).then(function(ResUser) {
        self.ResUser = ResUser;
        if (ResUser.length > 1) {
          self.$el.find('#current-user').append('<option value="9999999">Todos los usuarios</option>');
          _.each(ResUser, function(item) {
            self.$el.find('#current-user').append('<option value="' + item.id + '">' + item.name + '</option>');
          });
        } else {
          self.$el.find('.user').css('display', 'none');
        }
        self.fecthCheckType();
        return self.fetchTaskType();
      }).then(function(TaskType) {
        self.TaskType = TaskType;
        if (TaskType.length > 1) {
          self.$el.find('#current-task-type').append('<option value="9999999">Todos los tipos</option>');
          _.each(TaskType, function(item) {
            self.$el.find('#current-task-type').append('<option value="' + item.id + '">' + item.name + '</option>');
          });
        } else {
          self.$el.find('.task-type').css('display', 'none');
        }
        return self.fetchCrmLead();
      }).then(function(CrmLead) {
        self.CrmLead = CrmLead;
        if (CrmLead.length > 1) {
          self.$el.find('#current-opportunity').append('<option value="9999999">Todas las oportunidades</option>');
          _.each(CrmLead, function(item) {
            self.$el.find('#current-opportunity').append('<option value="' + item.id + '">' + item.name + '</option>');
          });
        } else {
          self.$el.find('.opportunity').css('display', 'none');
        }
      });
      self.$el.find('#generate').css('display', 'inline');
      return;
    },

    fetchGenerate: function() {
      var self = this;
      self.$el.find('.search-form').block({
        message: null,
        overlayCSS: {
          backgroundColor: '#FAFAFA'
        }
      });
      self.$el.find('.report-form').block({
        message: null,
        overlayCSS: {
          backgroundColor: '#FAFAFA'
        }
      });

      this.fetchCrmTask().then(function(ResCrmTask) {
        return ResCrmTask;
      }).then(function(ResCrmTask) {
        self.ResCrmTask = ResCrmTask;
        return self.BuildTable();
      });
    },

    fecthIrModuleModule: function() {
      var self = this;
      var defer = $.Deferred();
      var fields = ['name', 'id'];
      var domain = [
        ['state', '=', 'installed'],
        ['name', 'in', self.modules]
      ];
      var IrModuleModule = new model.web.Model('ir.module.module');
      IrModuleModule.query(fields).filter(domain).all().then(function(results) {
        defer.resolve(results);
      })
      return defer;
    },

    fecthCheckType: function() {
      var self = this;
      var modules = self.checkModel('eiru_crm');
      if (modules.length == 0) {
        self.$el.find('.type').css('display', 'none');
      }
    },

    fetchResUser: function() {
      var self = this;
      var defer = $.Deferred();
      var fields = ['id', 'name', 'store_id'];
      var ResUser = new model.web.Model('res.users');
      ResUser.query(fields).filter().all().then(function(results) {
        defer.resolve(results);
      });
      return defer;
    },

    fetchResStore: function() {
      var self = this;
      var defer = $.Deferred();
      var field = ['id', 'name', 'company_id'];
      var ResStore = new model.web.Model('res.store');
      ResStore.query(field).all().then(function(results) {
        defer.resolve(results);
      });
      return defer;
    },

    fetchResCompany: function() {
      var self = this;
      var defer = $.Deferred();
      var currency = new model.web.Model('res.company');
      var field = ['id', 'name', 'currency_id', 'logo'];
      currency.query(field).filter().all().then(function(results) {
        defer.resolve(results);
      });
      return defer;
    },

    getResCompany: function (id) {
      var self = this;
      return _.filter(self.ResCompany,function (item) {
        return item.id == id;
      })
    },

    fetchCrmTask: function() {
      var self = this;
      var defer = $.Deferred();
      var company = self.$el.find('#current-company').val();
      var store = self.$el.find('#current-store').val();
      var user = self.$el.find('#current-user').val();
      var state = self.$el.find('#current-state').val();
      var task_type = self.$el.find('#current-task-type').val();
      var opportunity = self.$el.find('#current-opportunity').val();
      var date = self.$el.find('#current-date').val()
      var desde = self.$el.find('#from').val();
      var hasta = self.$el.find('#to').val();
      var domain = [
        ['active', '=', true]
      ];

      if (company && company != 9999999) {
        var opportunity_ids = _.map(_.filter(self.CrmLead, function(item) {
          return item.company_id[0] == company;
        }), function(map) {
          return map.id;
        });
        domain.push(['opportunity_id', 'in', opportunity_ids]);
      }

      if (store && store != 9999999) {
        var user_ids = _.map(_.filter(self.ResUser, function(item) {
          return item.store_id[0] == store;
        }), function(map) {
          return map.id;
        });
        domain.push(['user_id', 'in', user_ids]);
      }
      if (user && user != 9999999) {

        domain.push(['user_id', '=', parseInt(user)]);
      }

      if (state != 9999999) {
        domain.push(['state', '=', state]);
      }

      if (task_type && task_type != 9999999) {
        domain.push(['task_type_id', '=', parseInt(task_type)]);
      }

      if (opportunity && opportunity != 9999999) {
        domain.push(['opportunity_id', '=', parseInt(opportunity)]);
      }
      if (date && date != 9999999) {

        if (desde) {
          var date = desde.split('/')
          date = (date[2] + "-" + date[1] + "-" + date[0]);
          domain.push(['date_action_next', '>=', date]);
        }

        if (hasta) {
          var date = hasta.split('/')
          date = (date[2] + "-" + date[1] + "-" + date[0]);
          domain.push(['date_action_next', '<=', date]);
        }

        if (date == 'today') {
          var today = moment().format('YYYY-MM-DD');
          domain.push(['date_action_next', '=', today]);
        }

        if (date == 'yesterday') {
          var yesterday = moment().add(-1, 'days').format('YYYY-MM-DD');
          domain.push(['date_action_next', '=', yesterday]);
        }

        if (date == 'currentMonth') {
          var currentMonth = moment().format('YYYY-MM');
          domain.push(['date_action_next', 'like', currentMonth]);
        }

        if (date == 'lastMonth') {
          var lastMonth = moment().add(-1, 'months').format('YYYY-MM');
          domain.push(['date_action_next', 'like', lastMonth]);
        }
      }
      var fields = [
        'id',
        'partner_id',
        'name',
        'description',
        'create_date',
        'user_id',
        'task_type_id',
        'opportunity_id',
        'state',
        'date_action_next',
      ];
      var ResCrmTask = new model.web.Model('crm.task');
      ResCrmTask.query(fields).filter(domain).all().then(function(results) {
        defer.resolve(results);
      });
      return defer;
    },

    fetchTaskType: function() {
      var self = this;
      var defer = $.Deferred();

      var TaskType = new model.web.Model('crm.task.type');
      TaskType.query(['id', 'name']).filter([
        ['active', '=', true]
      ]).all().then(function(results) {
        defer.resolve(results);
      });
      return defer;
    },

    fetchCrmLead: function() {
      var self = this;
      var defer = $.Deferred();

      var CrmLead = new model.web.Model('crm.lead');
      CrmLead.query(['id', 'partner_id', 'name', 'description', 'create_date', 'user_id', 'stage_id', 'type']).filter([
        ['active', '=', true]
      ]).all().then(function(results) {
        defer.resolve(results);
      });

      return defer;
    },

    fetchResPartner: function() {
      var self = this;
      var defer = $.Deferred();
      var ResPartner = new model.web.Model('res.partner');
      ResPartner.query(['id', 'name', 'ruc']).filter([
        ['active', '=', true]
      ]).all().then(function(results) {
        defer.resolve(results);
      });
      return defer;
    },
    updateUser: function() {
      var self = this;
      var store = self.$el.find('#current-store').val();
      if (store != 9999999) {
        var user = self.$el.find('#current-user').empty();
        self.$el.find('#current-user').append('<option value="9999999">Todos los usuarios</option>');
        _.each(self.ResUser, function(item) {
          if (parseFloat(store) == item.store_id[0]) {
            self.$el.find('#current-user').append('<option value="' + item.id + '">' + item.name + '</option>');
          }
        });
      } else {
        var user = self.$el.find('#current-user').empty();
        self.$el.find('#current-user').append('<option value="9999999">Todos los usuarios</option>');
        _.each(self.ResUser, function(item) {
          self.$el.find('#current-user').append('<option value="' + item.id + '">' + item.name + '</option>');
        });
      }
    },

    BuildTable: function() {
      var self = this;
      var data = [];

      var crmtask = self.ResCrmTask;
      _.each(crmtask, function(item) {
        if (item.state == 'pending') {
          var state = 'Pendiente';
        }
        if (item.state == 'done') {
          var state = 'Realizado';
        }
        if (item.state == 'cancel') {
          var state = 'Cancelado';
        }

        data.push({
          id: item.id,
          partner_id: self.valorNull(item.partner_id[0]),

          create_date: moment(item.create_date).format("DD/MM/YYYY"),
          user: item.user_id[1],
          partner: self.valorNull(item.partner_id[1]),
          activity: self.valorNull(item.name),
          description: self.valorNull(item.description),
          opportunity_name: self.valorNull(item.opportunity_id[1]),
          next_date: moment(item.date_action_next).format("DD/MM/YYYY"),
          task_name: item.task_type_id[1],
          state: state,
        });
      });
      self.content = data;
      self.loadTable(data);
      self.$el.find('.report-form').css('display', 'block');
      self.$el.find('.search-form').unblock();
      self.$el.find('.report-form').unblock();
    },

    loadTable: function(rowsTable) {
      var self = this;
      self.rowsData = rowsTable;
      var table = this.$el.find('#table');
      table.bootstrapTable('load', rowsTable);
    },

    clickOnAction: function(e) {
      var self = this;
      var action = self.$el.find(e.target).val();
      var company = $('#current-company').val();
      if (company && company != 9999999) {
        var ResCompany = self.getResCompany(company).shift();

      } else {
        var ResCompany = self.ResCompany[0];
      }
      var getColumns = [];
      var rows = [];
      var table = self.$el.find("#table");
      var column = table.bootstrapTable('getVisibleColumns');
      var row = table.bootstrapTable('getData');

      row.push({});
      if (action === 'pdf') {
        var data = _.map(column, function(val) {
          return val.field
        });
        _.each(_.map(column, function(val) {
          return val
        }), function(item) {
          getColumns.push([{
            title: item.title,
            dataKey: item.field
          }]);
        });
        var pdf_title = 'Histórico de Tareas';
        var pdf_type = 'l';
        var pdf_name = 'historico_de_tareas_';
        var pdf_columnStyles = {
          create_date: {columnWidth: 16,halign: 'center'},
          user: {columnWidth: 30,halign: 'left'},
          partner: {columnWidth: 30,halign: 'left'},
          activity: {columnWidth: 30,halign: 'left'},
          description: {halign: 'left'},
          opportunity_name: {columnWidth: 25,halign: 'left'},
          next_date: {columnWidth: 18,halign: 'center'},
          task_name: {columnWidth: 16,halign: 'left'},
          state: {columnWidth: 16,halign: 'center'},
        };

        var filter = self.getFilter();
        var pdf = new model.eiru_reports.ReportPdfWidget(self);
        pdf.drawPDF(
          _.flatten(getColumns),
          row,
          ResCompany,
          pdf_title,
          pdf_type,
          pdf_name,
          pdf_columnStyles,
          filter,
        );
      }
    },
    getFilter: function() {
      var self = this;
      var company = self.$el.find('#current-company').val();
      var store = self.$el.find('#current-store').val();
      var user = self.$el.find('#current-user').val();
      var state = self.$el.find('#current-state').val();
      var task_type = self.$el.find('#current-task-type').val();
      var opportunity = self.$el.find('#current-opportunity').val();
      var date = self.$el.find('#current-date').val();
      var desde = self.$el.find('#from').val();
      var hasta = self.$el.find('#to').val();

      var filter = [];

      if (company && company != 9999999) {
        var ResCompany = _.filter(self.ResCompany, function(item) {
          return item.id == company;
        });
        filter.push({
          title: 'Empresa',
          value: ResCompany[0].name,
        });
      }

      if (store && store != 9999999) {
        var ResStore = _.filter(self.ResStore, function(item) {
          return item.id == store;
        });

        filter.push({
          title: 'Sucursal',
          value: ResStore[0].name,
        });

      }
      if (user && user != 9999999) {
        var ResUser = _.filter(self.ResUser, function(item) {
          return item.id == user;
        })
        filter.push({
          title: 'Responsable',
          value: ResUser[0].name,
        });
      }

      if (state && state != 9999999) {
        filter.push({
          title: 'Estado',
          value: $('#current-state option:selected').text(),
        });
      }

      if (task_type && task_type != 9999999) {
        var TaskType = _.filter(self.TaskType, function(item) {
          return item.id == task_type;
        })
        filter.push({
          title: 'Tipo de Actividad',
          value: TaskType[0].name,
        });
      }

      if (opportunity && opportunity != 9999999) {
        var CrmLead = _.filter(self.CrmLead, function(item) {
          return item.id == opportunity;
        })
        filter.push({
          title: 'Oportunidad',
          value: CrmLead[0].name,
        });
      }

      if (date && date != 9999999) {
        moment.locale('es', {
          months: 'Enero_Febrero_Marzo_Abril_Mayo_Junio_Julio_Agosto_Septiembre_Octubre_Noviembre_Diciembre'.split('_'),
        });

        if (date == 'range') {
          filter.push({
            title: 'Fecha',
            value: desde + ' al ' + hasta,
          });

        } else {
          if (date == 'today') {
            var fecha = moment().format('DD/MM/YYYY');
          }
          if (date == 'yesterday') {
            var fecha = moment().add(-1, 'days').format('DD/MM/YYYY');
          }
          if (date == 'currentMonth') {
            var fecha = moment().format('MMMM/YYYY');
          }
          if (date == 'lastMonth') {
            var fecha = moment().add(-1, 'months').format('MMMM/YYYY');
          }
          filter.push({
            title: 'Fecha',
            value: fecha,
          });
        }
      }
      return filter;
    },
  });
}
